import { FromCognitoIdentityPoolParameters } from '@aws-sdk/credential-provider-cognito-identity/dist-types/fromCognitoIdentityPool';
import { Credentials } from '@aws-sdk/types/dist-types/credentials';

type StateType = 'Task' | 'Parallel' | 'Map' | 'Pass' | 'Wait' | 'Choice' | 'Succeed' | 'Fail';

interface BaseState {
    Type: StateType;
    Comment?: string;
}

type JSONPrimitiveValue = null | boolean | number | string;
type JSONArray = (JSONPrimitiveValue | JSONArray | JSONObject)[];
type JSONObject = {
    [key: string]: JSONPrimitiveValue | JSONObject | JSONArray;
};
type JSONValue = JSONPrimitiveValue | JSONObject | JSONArray;

type PayloadTemplate = JSONObject;
interface CanHaveInputPath {
    InputPath?: string | null;
}
interface CanHaveParameters {
    Parameters?: PayloadTemplate;
}
interface CanHaveResultSelector {
    ResultSelector?: PayloadTemplate;
}
interface CanHaveResultPath {
    ResultPath?: string | null;
}
interface CanHaveOutputPath {
    OutputPath?: string | null;
}

type StringOperatorNames = 'StringEquals' | 'StringLessThan' | 'StringGreaterThan' | 'StringLessThanEquals' | 'StringGreaterThanEquals' | 'StringMatches';
type StringPathOperatorNames = 'StringEqualsPath' | 'StringLessThanPath' | 'StringGreaterThanPath' | 'StringLessThanEqualsPath' | 'StringGreaterThanEqualsPath';
type NumericOperatorNames = 'NumericEquals' | 'NumericLessThan' | 'NumericGreaterThan' | 'NumericLessThanEquals' | 'NumericGreaterThanEquals';
type NumericPathOperatorNames = 'NumericEqualsPath' | 'NumericLessThanPath' | 'NumericGreaterThanPath' | 'NumericLessThanEqualsPath' | 'NumericGreaterThanEqualsPath';
type BooleanOperatorNames = 'BooleanEquals';
type BooleanPathOperatorNames = 'BooleanEqualsPath';
type TimestampOperatorNames = 'TimestampEquals' | 'TimestampLessThan' | 'TimestampGreaterThan' | 'TimestampLessThanEquals' | 'TimestampGreaterThanEquals';
type TimestampPathOperatorNames = 'TimestampEqualsPath' | 'TimestampLessThanPath' | 'TimestampGreaterThanPath' | 'TimestampLessThanEqualsPath' | 'TimestampGreaterThanEqualsPath';
type TypeTestOperatorNames = 'IsNull' | 'IsPresent' | 'IsNumeric' | 'IsString' | 'IsBoolean' | 'IsTimestamp';
type StringComparisonOperator = {
    [P in StringOperatorNames]?: string;
};
type StringPathComparisonOperator = {
    [P in StringPathOperatorNames]?: string;
};
type NumericComparisonOperator = {
    [P in NumericOperatorNames]?: number;
};
type NumericPathComparisonOperator = {
    [P in NumericPathOperatorNames]?: string;
};
type BooleanComparisonOperator = {
    [P in BooleanOperatorNames]?: boolean;
};
type BooleanPathComparisonOperator = {
    [P in BooleanPathOperatorNames]?: string;
};
type TimestampComparisonOperator = {
    [P in TimestampOperatorNames]?: string;
};
type TimestampPathComparisonOperator = {
    [P in TimestampPathOperatorNames]?: string;
};
type TypeTestComparisonOperator = {
    [P in TypeTestOperatorNames]?: true;
};
type ComparisonOperator = StringComparisonOperator | StringPathComparisonOperator | NumericComparisonOperator | NumericPathComparisonOperator | BooleanComparisonOperator | BooleanPathComparisonOperator | TimestampComparisonOperator | TimestampPathComparisonOperator | TypeTestComparisonOperator;
type BaseDataTestExpression = {
    Variable: string;
};
type DataTestExpression = BaseDataTestExpression & ComparisonOperator;
type BooleanExpression = {
    And?: ChoiceRuleWithoutNext[];
    Or?: ChoiceRuleWithoutNext[];
    Not?: ChoiceRuleWithoutNext;
};
type ChoiceRuleWithoutNext = DataTestExpression | BooleanExpression;
type ChoiceRule = (DataTestExpression | BooleanExpression) & {
    Next: string;
};
interface BaseChoiceState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Choice';
    Choices: ChoiceRule[];
    Default?: string;
}
type ChoiceState = BaseChoiceState;

interface EndableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    End: true;
}
interface SucceedOrFailState extends BaseState {
    Type: Extract<StateType, 'Succeed' | 'Fail'>;
}
type TerminalState = EndableState | SucceedOrFailState;

interface BaseFailState extends BaseState {
    Type: 'Fail';
    Cause?: string;
    Error?: string;
}
type FailState = TerminalState & BaseFailState;

type Retrier = {
    ErrorEquals: string[];
    IntervalSeconds?: number;
    MaxAttempts?: number;
    BackoffRate?: number;
};
interface RetryableState {
    Retry?: Retrier[];
}
type Catcher = {
    ErrorEquals: string[];
    Next: string;
    ResultPath?: string;
};
interface CatchableState {
    Catch?: Catcher[];
}

interface NextableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    Next: string;
}
type IntermediateState = NextableState;

interface BaseMapState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath, RetryableState, CatchableState {
    Type: 'Map';
    Iterator: Omit<StateMachineDefinition, 'Version' | 'TimeoutSeconds'>;
    ItemsPath?: string;
    MaxConcurrency?: number;
}
type MapState = (IntermediateState | TerminalState) & BaseMapState;

interface BaseParallelState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath, RetryableState, CatchableState {
    Type: 'Parallel';
    Branches: Omit<StateMachineDefinition, 'Version' | 'TimeoutSeconds'>[];
}
type ParallelState = (IntermediateState | TerminalState) & BaseParallelState;

interface BasePassState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Pass';
    Result?: JSONValue;
}
type PassState = (IntermediateState | TerminalState) & BasePassState;

interface BaseSucceedState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Succeed';
}
type SucceedState = TerminalState & BaseSucceedState;

interface BaseTaskState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath, RetryableState, CatchableState {
    Type: 'Task';
    Resource: string;
}
type TaskState = (IntermediateState | TerminalState) & BaseTaskState;

interface BaseWaitState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Wait';
    Seconds?: number;
    Timestamp?: string;
    SecondsPath?: string;
    TimestampPath?: string;
}
type WaitState = (IntermediateState | TerminalState) & BaseWaitState;

type AllStates = TaskState | ParallelState | MapState | PassState | WaitState | ChoiceState | SucceedState | FailState;

interface StateMachineDefinition {
    States: Record<string, AllStates>;
    StartAt: string;
    Comment?: string;
    Version?: string;
    TimeoutSeconds?: number;
}

type TaskStateResourceLocalHandler = {
    [taskStateName: string]: (input: JSONValue) => Promise<JSONValue>;
};
type WaitStateTimeOverride = {
    [waitStateName: string]: number;
};
interface Overrides {
    taskResourceLocalHandlers?: TaskStateResourceLocalHandler;
    waitTimeOverrides?: WaitStateTimeOverride;
}
interface ValidationOptions {
    readonly checkPaths?: boolean;
    readonly checkArn?: boolean;
}
interface AWSConfig {
    region: string;
    credentials?: {
        cognitoIdentityPool?: FromCognitoIdentityPoolParameters;
        accessKeys?: Omit<Credentials, 'expiration'>;
    };
}
interface StateMachineOptions {
    validationOptions?: ValidationOptions;
    awsConfig?: AWSConfig;
}
interface RunOptions {
    overrides?: Overrides;
    noThrowOnAbort?: boolean;
}

declare class StateMachine {
    /**
     * The structure of the State Machine as represented by the Amazon States Language.
     */
    private readonly definition;
    /**
     * Options to control certain settings of the state machine.
     */
    private readonly stateMachineOptions;
    /**
     * Constructs a new state machine.
     * @param definition The state machine definition defined using the Amazon States Language (https://states-language.net/spec.html).
     * @param stateMachineOptions Options to control certain settings of the state machine.
     * These options also apply to state machines defined in the `Iterator` field of `Map` states and in the `Branches` field of `Parallel` states.
     */
    constructor(definition: StateMachineDefinition, stateMachineOptions?: StateMachineOptions);
    /**
     * Executes the state machine, running through the states specified in the definition.
     * If the execution fails, the result will throw an `ExecutionError` explaining why the
     * execution failed.
     *
     * By default, if the execution is aborted, the result will throw an `ExecutionAbortedError`. This behavior can be changed by setting
     * the `noThrowOnAbort` option to `true`, in which case the result will be `null`.
     *
     * @param input The input to pass to this state machine execution.
     * @param options Miscellaneous options to control certain behaviors of the execution.
     */
    run(input: JSONValue, options?: RunOptions): {
        abort: () => void;
        result: Promise<JSONValue>;
    };
    /**
     * Helper method that handles the execution of the machine states and the transitions between them.
     */
    private execute;
}

/**
 * Represents the failure of a state machine execution.
 */
declare class ExecutionError extends Error {
    #private;
    constructor(caughtError: Error);
    get getWrappedError(): Error;
}

/**
 * Represents the abortion of a state machine execution.
 */
declare class ExecutionAbortedError extends Error {
    constructor();
}

/**
 * Base class for all internal errors that can be thrown during the state machine execution.
 */
declare class RuntimeError extends Error {
    /**
     * Whether this runtime error can be matched in a `Retry` field
     */
    protected retryable: boolean;
    /**
     * Whether this runtime error can be caught in a `Catch` field
     */
    protected catchable: boolean;
    constructor(message: string);
    get isRetryable(): boolean;
    get isCatchable(): boolean;
}

declare class StatesTimeoutError extends RuntimeError {
    constructor();
}

export { ExecutionAbortedError, ExecutionError, StatesTimeoutError as ExecutionTimeoutError, StateMachine };
